mNameplates = LibStub("AceAddon-3.0"):NewAddon("mNameplates", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
local mNameplates = mNameplates
MDB_MN = mNameplates


local UIF
local LSM = LibStub("LibSharedMedia-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local Nameplates = LibStub("LibNameplate-1.0")

local db

local bossIconTexture = [[Interface\TargetingFrame\UI-TargetingFrame-Skull]]

local ResetTextures = {
	ThreatGlow = [[Interface\TargetingFrame\UI-TargetingFrame-Flash]],
	Highlight = [[Interface\Tooltips\Nameplate-Glow]],
	BossIcon = [[Interface\TargetingFrame\UI-TargetingFrame-Skull]],
	ElitIcon = [[Interface\Tooltips\EliteNameplateIcon]],
	CastBorder = [[Interface\Tooltips\Nameplate-Border]],
	RaidIcon = [[nterface\TargetingFrame\UI-RaidTargetingIcons]],
}

function mNameplates:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
		}
	}
	
	self.db  = LibStub("AceDB-3.0"):New("mNameplatesDB", defaults, "Default")
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	self:SetupLDB()
	
	self:SetEnabledState(db.Enabled)
	
	self.Callbacks = LibStub("CallbackHandler-1.0"):New(self)	
	
	self.Plates = {}

	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mNameplates", {font = LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"), fontSize = 16})
end

function mNameplates:OnEnable()		
	self:RegisterChatCommand("mn", "ToggleConfig")
	
	Nameplates.RegisterCallback(self, "LibNameplate_FoundGUID")
	Nameplates.RegisterCallback(self, "LibNameplate_NewNameplate")
	
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	
	for i, plate in pairs({select(2, Nameplates:GetAllNameplates())}) do
		self:LibNameplate_NewNameplate("LibNameplate_NewNameplate", plate)
	end
end

function mNameplates:OnDisable()
	for	i, plate in pairs(self.Plates) do
		plate.DefaultItems.ThreatGlow:SetTexture(ResetTextures.ThreatGlow)
		plate.DefaultItems.Highlight:SetTexture(ResetTextures.Highlight)
		plate.DefaultItems.BossIcon:SetTexture(ResetTextures.BossIcon)
		plate.DefaultItems.ElitIcon:SetTexture(ResetTextures.ElitIcon)
		plate.DefaultItems.CastBorder:SetTexture(ResetTextures.CastBorder)
		plate.DefaultItems.RaidIcon:SetTexture(ResetTextures.RaidIcon)
		
		plate.DefaultItems.Name:Show()
		plate.DefaultItems.HealthBorder:Show()
		plate.DefaultItems.Level:Show()
		
		plate.DefaultItems = nil
		plate.mn = nil
	end
	wipe(self.Plates)
	
	Nameplates.UnregisterAllCallbacks(self)
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:LibNameplate_NewNameplate(event, plate)
	self:PrepPlate(plate)
	plate.DefaultItems.Level:Hide()
	
	self.Callbacks:Fire("PlateShow", plate)
end

function mNameplates:LibNameplate_FoundGUID(event, plate, gUID, unit)	
	self.Callbacks:Fire("PlateGUID", gUID, unit)
end

function mNameplates:ProfileChanged(event)
	db = self.db.profile
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Events ~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:UPDATE_MOUSEOVER_UNIT(...)
	local plate = Nameplates:GetNameplateByUnit("mouseover")
	if plate then
		self.Callbacks:Fire("PlateMouuseOver", plate)
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Creation ~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
do
	local regions = {}
	function mNameplates:PrepPlate(plate)	
		if plate.mn then return end
		local healthBar, castBar = plate:GetChildren()
		local threatGlow, healthBorder, castbarBorder, castNoInterupt, castIcon, highlight, name, level, bossIcon, raidIcon, eliteIcon = plate:GetRegions()		
		healthBar:Hide()
		castBar:Hide()
		
		threatGlow:SetTexture(nil)
		highlight:SetTexture(nil)
		bossIcon:SetTexture(nil)
		eliteIcon:SetTexture(nil)
		castbarBorder:SetTexture(nil)
		castNoInterupt:SetTexture(nil)		
		raidIcon:SetTexture(nil)
		
		name:Hide()		
		healthBorder:Hide()
		
		plate.mn = {}
		
		plate.DefaultItems = {
			HealthBar = healthBar,
			CastBar = castBar,
			ThreatGlow = threatGlow,
			HealthBorder = healthBorder,
			CastBorder = castbarBorder,
			CastNoInterupt = castNoInterupt,
			Icon = castIcon,
			Highlight = highlight,
			Name = name,
			Level = level,
			BossIcon = bossIcon,
			RaidIcon = raidIcon,
			ElitIcon = eliteIcon,
		}	

		tinsert(self.Plates, plate)
		
		self.Callbacks:Fire("PlateNew", plate)
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Settings ~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:Applyettings()
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Misc ~~~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:IteratePlates()
	return pairs(self.Plates)
end

function mNameplates:GetExtra(plate)
	return plate.mn
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Config ~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
do	
	local function Callback(widget, event, ...)
	end

	local OptionsFrame
	local SelectedOption
	local OptionsStatus = {}
	
	function mNameplates:ToggleConfig()
		if not OptionsFrame then	
			OptionsFrame = UIF:Frame("mNameplates")
			OptionsFrame:SetLayout("Fill")
			OptionsFrame:SetWidth(813)
			OptionsFrame:SetHeight(630)
			OptionsFrame:Hide()			
			OptionsFrame:SetCallback("OnClose", function(widget)				
				OptionsFrame:ReleaseChildren()		
				OptionsFrame.frame:Hide()
			end)			
		end
		
		if OptionsFrame.frame:IsShown() then
			OptionsFrame:Hide()
			return
		else			
			m = UIF:TreeGroup()
			m:SetLayout("Fill")	
			m:SetCallback("OnGroupSelected", function(self, event, group) SelectedOption = group end)									
			OptionsFrame:AddChild(m)
			
			m:AddGroup({{text = "Main", value = "Main"}}, function()
				local g = UIF:InlineGroup1()
				
				return g
			end)
		
			for moduleName, module in self:IterateModules() do
				if module.GetOptions then
					local options = module:GetOptions()
					if options and type(options) == "table" then
						for i, option in pairs(options) do
							m:AddGroup(option.data, option.func)
						end
					end
				end
			end
			
			m:AddGroup({{text = "Profile", value = "Profile"}}, function()
				return UIF:AceProfileOption(mNameplates.db)
			end)
			
			m:SetSelected(SelectedOption or "Main")
			m:SetStatusTable(OptionsStatus)	
			OptionsFrame:Show()
		end		
	end

	function mNameplates:SetupLDB()
		local LDB = LibStub("LibDataBroker-1.1")
		if not LDB then return end
		
		local l = LDB:NewDataObject("mNameplates")
		l.type = "launcher"
		l.icon = [[Interface\Icons\Spell_DeathKnight_RuneTap]]
		l.tocname = "mNameplates"
		l.label = "mNameplates"
		l.OnClick = function(self, button)
			mNameplates:ToggleConfig()
		end
		l.OnTooltipShow = function(tt)
			tt:AddLine("mNameplates")
			tt:AddLine("Click to toggle configuration")
		end
		
		self.ldbojb = l
	end
end